import { Behavior, ElementStyles, FASTElement } from "@microsoft/fast-element";
declare type MediaQueryListListener = (this: MediaQueryList, ev?: MediaQueryListEvent) => any;
interface MatchMediaStyleSheetBehavior extends Behavior {
    query: MediaQueryList;
    cache: WeakMap<typeof FASTElement, MediaQueryListListener | Array<MediaQueryListListener>>;
    sheet: ElementStyles;
    constructListener: (source: typeof FASTElement, sheet: ElementStyles) => MediaQueryListListener;
}
/**
 * Construct a behavior factory that will conditionally apply a stylesheet based
 * on a MediaQueryList
 *
 * @param query - The MediaQueryList to subscribe to matches for.
 *
 * @public
 */
export declare function matchMediaStylesheetBehaviorFactory(query: MediaQueryList): (sheet: ElementStyles) => Readonly<{
    query: MediaQueryList;
    cache: WeakMap<(new () => HTMLElement & FASTElement) & {
        from<TBase extends {
            new (): HTMLElement;
            prototype: HTMLElement;
        }>(BaseType: TBase): new () => InstanceType<TBase> & FASTElement;
        define<TType extends Function>(Type: TType, nameOrDef?: string | import("@microsoft/fast-element").PartialFASTElementDefinition | undefined): TType;
        getDefinition<T extends Function>(Type: T): import("@microsoft/fast-element").FASTElementDefinition | undefined;
    }, ((this: MediaQueryList) => void)[] | ((this: MediaQueryList) => void)>;
    sheet: ElementStyles;
    constructListener(this: MatchMediaStyleSheetBehavior, source: (new () => HTMLElement & FASTElement) & {
        from<TBase extends {
            new (): HTMLElement;
            prototype: HTMLElement;
        }>(BaseType: TBase): new () => InstanceType<TBase> & FASTElement;
        define<TType extends Function>(Type: TType, nameOrDef?: string | import("@microsoft/fast-element").PartialFASTElementDefinition | undefined): TType;
        getDefinition<T extends Function>(Type: T): import("@microsoft/fast-element").FASTElementDefinition | undefined;
    }, sheet: ElementStyles): MediaQueryListListener;
    bind(this: MatchMediaStyleSheetBehavior, source: (new () => HTMLElement & FASTElement) & {
        from<TBase extends {
            new (): HTMLElement;
            prototype: HTMLElement;
        }>(BaseType: TBase): new () => InstanceType<TBase> & FASTElement;
        define<TType extends Function>(Type: TType, nameOrDef?: string | import("@microsoft/fast-element").PartialFASTElementDefinition | undefined): TType;
        getDefinition<T extends Function>(Type: T): import("@microsoft/fast-element").FASTElementDefinition | undefined;
    }): void;
    unbind(this: MatchMediaStyleSheetBehavior, source: (new () => HTMLElement & FASTElement) & {
        from<TBase extends {
            new (): HTMLElement;
            prototype: HTMLElement;
        }>(BaseType: TBase): new () => InstanceType<TBase> & FASTElement;
        define<TType extends Function>(Type: TType, nameOrDef?: string | import("@microsoft/fast-element").PartialFASTElementDefinition | undefined): TType;
        getDefinition<T extends Function>(Type: T): import("@microsoft/fast-element").FASTElementDefinition | undefined;
    }): void;
}>;
/**
 * Implements {@link matchMediaStylesheetBehaviorFactory} for the {@link https://developer.mozilla.org/en-US/docs/Web/CSS/@media/forced-colors | forced-colors} query.
 * This can be used to construct a behavior to apply a forced-colors only stylesheet.
 * @public
 */
export declare const forcedColorsStylesheetBehavior: (sheet: ElementStyles) => Readonly<{
    query: MediaQueryList;
    cache: WeakMap<(new () => HTMLElement & FASTElement) & {
        from<TBase extends {
            new (): HTMLElement;
            prototype: HTMLElement;
        }>(BaseType: TBase): new () => InstanceType<TBase> & FASTElement;
        define<TType extends Function>(Type: TType, nameOrDef?: string | import("@microsoft/fast-element").PartialFASTElementDefinition | undefined): TType;
        getDefinition<T extends Function>(Type: T): import("@microsoft/fast-element").FASTElementDefinition | undefined;
    }, ((this: MediaQueryList) => void)[] | ((this: MediaQueryList) => void)>;
    sheet: ElementStyles;
    constructListener(this: MatchMediaStyleSheetBehavior, source: (new () => HTMLElement & FASTElement) & {
        from<TBase extends {
            new (): HTMLElement;
            prototype: HTMLElement;
        }>(BaseType: TBase): new () => InstanceType<TBase> & FASTElement;
        define<TType extends Function>(Type: TType, nameOrDef?: string | import("@microsoft/fast-element").PartialFASTElementDefinition | undefined): TType;
        getDefinition<T extends Function>(Type: T): import("@microsoft/fast-element").FASTElementDefinition | undefined;
    }, sheet: ElementStyles): MediaQueryListListener;
    bind(this: MatchMediaStyleSheetBehavior, source: (new () => HTMLElement & FASTElement) & {
        from<TBase extends {
            new (): HTMLElement;
            prototype: HTMLElement;
        }>(BaseType: TBase): new () => InstanceType<TBase> & FASTElement;
        define<TType extends Function>(Type: TType, nameOrDef?: string | import("@microsoft/fast-element").PartialFASTElementDefinition | undefined): TType;
        getDefinition<T extends Function>(Type: T): import("@microsoft/fast-element").FASTElementDefinition | undefined;
    }): void;
    unbind(this: MatchMediaStyleSheetBehavior, source: (new () => HTMLElement & FASTElement) & {
        from<TBase extends {
            new (): HTMLElement;
            prototype: HTMLElement;
        }>(BaseType: TBase): new () => InstanceType<TBase> & FASTElement;
        define<TType extends Function>(Type: TType, nameOrDef?: string | import("@microsoft/fast-element").PartialFASTElementDefinition | undefined): TType;
        getDefinition<T extends Function>(Type: T): import("@microsoft/fast-element").FASTElementDefinition | undefined;
    }): void;
}>;
export {};
