import { __decorate } from "tslib";
import { attr, DOM, FASTElement, observable } from "@microsoft/fast-element";
import { Direction, RtlScrollConverter } from "@microsoft/fast-web-utilities";
var AnchoredRegionHorizontalPositionLabel;
(function (AnchoredRegionHorizontalPositionLabel) {
    AnchoredRegionHorizontalPositionLabel["left"] = "left";
    AnchoredRegionHorizontalPositionLabel["insetLeft"] = "insetLeft";
    AnchoredRegionHorizontalPositionLabel["insetRight"] = "insetRight";
    AnchoredRegionHorizontalPositionLabel["right"] = "right";
    AnchoredRegionHorizontalPositionLabel["undefined"] = "undefined";
})(AnchoredRegionHorizontalPositionLabel || (AnchoredRegionHorizontalPositionLabel = {}));
var AnchoredRegionVerticalPositionLabel;
(function (AnchoredRegionVerticalPositionLabel) {
    AnchoredRegionVerticalPositionLabel["top"] = "top";
    AnchoredRegionVerticalPositionLabel["insetTop"] = "insetTop";
    AnchoredRegionVerticalPositionLabel["insetBottom"] = "insetBottom";
    AnchoredRegionVerticalPositionLabel["bottom"] = "bottom";
    AnchoredRegionVerticalPositionLabel["undefined"] = "undefined";
})(AnchoredRegionVerticalPositionLabel || (AnchoredRegionVerticalPositionLabel = {}));
/**
 * location enum for transform origin settings
 */
var Location;
(function (Location) {
    Location["top"] = "top";
    Location["left"] = "left";
    Location["right"] = "right";
    Location["bottom"] = "bottom";
})(Location || (Location = {}));
export class AnchoredRegion extends FASTElement {
    constructor() {
        super();
        this.anchor = "";
        this.viewport = "";
        this.horizontalPositioningMode = "uncontrolled";
        this.horizontalDefaultPosition = "unset";
        this.horizontalInset = false;
        this.horizontalThreshold = "";
        this.horizontalScaling = false;
        this.verticalPositioningMode = "uncontrolled";
        this.verticalDefaultPosition = "unset";
        this.verticalInset = false;
        this.verticalThreshold = "";
        this.verticalScaling = false;
        this.regionStyle = "";
        /**
         * indicates that an initial positioning pass on layout has completed
         */
        this.initialLayoutComplete = false;
        this.anchorElement = null;
        this.viewportElement = null;
        this.openRequestAnimationFrame = false;
        this.currentDirection = Direction.ltr;
        /**
         * sets the starting configuration for component internal values
         */
        this.setInitialState = () => {
            this.initialLayoutComplete = false;
            this.transformOrigin = "top left";
            this.regionTop = "unset";
            this.regionRight = "unset";
            this.regionBottom = "unset";
            this.regionLeft = "unset";
            this.regionWidth = "fit-content";
            this.regionHeight = "fit-content";
            this.xTransformOrigin = Location.left;
            this.yTransformOrigin = Location.top;
            this.viewportRect = null;
            this.positionerDimension = { height: 0, width: 0 };
            this.anchorTop = 0;
            this.anchorRight = 0;
            this.anchorBottom = 0;
            this.anchorLeft = 0;
            this.anchorHeight = 0;
            this.anchorWidth = 0;
            this.viewportScrollTop = 0;
            this.viewportScrollLeft = 0;
            this.verticalPosition = AnchoredRegionVerticalPositionLabel.undefined;
            this.horizontalPosition = AnchoredRegionHorizontalPositionLabel.undefined;
            this.baseHorizontalOffset = 0;
            this.baseVerticalOffset = 0;
            this.regionStyle = "";
        };
        /**
         * connects observers and event handlers
         */
        this.connectObservers = () => {
            if (this.anchorElement === null || this.viewportElement === null) {
                return;
            }
            this.collisionDetector = new IntersectionObserver(this.handleCollision, {
                root: this.viewportElement,
                rootMargin: "0px",
                threshold: [0, 1],
            });
            this.collisionDetector.observe(this.region);
            this.collisionDetector.observe(this.anchorElement);
            this.resizeDetector = new window.ResizeObserver(this.handleResize);
            this.resizeDetector.observe(this.anchorElement);
            this.resizeDetector.observe(this.region);
            this.viewportElement.addEventListener("scroll", this.handleScroll);
        };
        /**
         * disconnect observers and event handlers
         */
        this.disconnectObservers = () => {
            // ensure the collisionDetector exists before disconnecting
            if (this.collisionDetector) {
                this.collisionDetector.disconnect();
            }
            // ensure the resizeDetector exists before disconnecting
            if (this.resizeDetector) {
                this.resizeDetector.disconnect();
            }
            this.disconnectViewport(this.viewportElement);
        };
        this.disconnectViewport = (viewport) => {
            if (viewport !== null) {
                viewport.removeEventListener("scroll", this.handleScroll);
            }
        };
        /**
         * Gets the viewport element by id, or defaults to component parent
         */
        this.getViewport = () => {
            if (typeof this.viewport !== "string") {
                return this.region.parentElement;
            }
            return document.getElementById(this.viewport);
        };
        /**
         *  Gets the anchor element by id
         */
        this.getAnchor = () => {
            return document.getElementById(this.anchor);
        };
        /**
         *  Handle collisions
         */
        this.handleCollision = (entries) => {
            let positionerRect = null;
            entries.forEach((entry) => {
                if (entry.target === this.region) {
                    this.handlePositionerCollision(entry, entries.length === 1);
                    positionerRect = entry.boundingClientRect;
                }
                else {
                    this.handleAnchorCollision(entry);
                }
            });
            if (this.viewportElement !== null) {
                this.viewportScrollTop = this.viewportElement.scrollTop;
                this.viewportScrollLeft = RtlScrollConverter.getScrollLeft(this.viewportElement, this.currentDirection);
            }
            if (entries.length === 2 && positionerRect !== null) {
                this.updatePositionerOffset(positionerRect);
            }
            this.requestLayoutUpdate();
        };
        /**
         *  Update data based on anchor collisions
         */
        this.handleAnchorCollision = (anchorEntry) => {
            this.viewportRect = anchorEntry.rootBounds;
            this.anchorTop = anchorEntry.boundingClientRect.top;
            this.anchorRight = anchorEntry.boundingClientRect.right;
            this.anchorBottom = anchorEntry.boundingClientRect.bottom;
            this.anchorLeft = anchorEntry.boundingClientRect.left;
            this.anchorHeight = anchorEntry.boundingClientRect.height;
            this.anchorWidth = anchorEntry.boundingClientRect.width;
        };
        /**
         *  Update data based on positioner collisions
         */
        this.handlePositionerCollision = (positionerEntry, shouldDeriveAnchorPosition) => {
            this.viewportRect = positionerEntry.rootBounds;
            const positionerRect = positionerEntry.boundingClientRect;
            this.positionerDimension = {
                height: positionerRect.height,
                width: positionerRect.width,
            };
            if (shouldDeriveAnchorPosition) {
                switch (this.verticalPosition) {
                    case AnchoredRegionVerticalPositionLabel.top:
                        this.anchorTop = positionerRect.bottom;
                        this.anchorBottom = this.anchorTop + this.anchorHeight;
                        break;
                    case AnchoredRegionVerticalPositionLabel.insetTop:
                        this.anchorBottom = positionerRect.bottom;
                        this.anchorTop = this.anchorBottom - this.anchorHeight;
                        break;
                    case AnchoredRegionVerticalPositionLabel.insetBottom:
                        this.anchorTop = positionerRect.top;
                        this.anchorBottom = this.anchorTop + this.anchorHeight;
                        break;
                    case AnchoredRegionVerticalPositionLabel.bottom:
                        this.anchorBottom = positionerRect.top;
                        this.anchorTop = this.anchorBottom - this.anchorHeight;
                        break;
                }
                switch (this.horizontalPosition) {
                    case AnchoredRegionHorizontalPositionLabel.left:
                        this.anchorLeft = positionerRect.right;
                        this.anchorRight = this.anchorLeft + this.anchorWidth;
                        break;
                    case AnchoredRegionHorizontalPositionLabel.insetLeft:
                        this.anchorRight = positionerRect.right;
                        this.anchorLeft = this.anchorRight - this.anchorWidth;
                        break;
                    case AnchoredRegionHorizontalPositionLabel.insetRight:
                        this.anchorLeft = positionerRect.left;
                        this.anchorRight = this.anchorLeft + this.anchorWidth;
                        break;
                    case AnchoredRegionHorizontalPositionLabel.right:
                        this.anchorRight = positionerRect.left;
                        this.anchorLeft = this.anchorRight - this.anchorWidth;
                        break;
                }
            }
        };
        /**
         *  Handle resize events
         */
        this.handleResize = (entries) => {
            entries.forEach((entry) => {
                if (entry.target === this.region) {
                    this.handleRegionResize(entry);
                }
                else {
                    this.handleAnchorResize(entry);
                }
            });
            this.requestLayoutUpdate();
        };
        /**
         *  Handle region resize events
         */
        this.handleRegionResize = (entry) => {
            // correct for rtl
            if (this.currentDirection === Direction.rtl) {
                this.baseHorizontalOffset =
                    this.baseHorizontalOffset -
                        (this.positionerDimension.width - entry.contentRect.width);
            }
            if (!this.horizontalScaling) {
                this.positionerDimension.width = entry.contentRect.width;
            }
            if (!this.verticalScaling) {
                this.positionerDimension.height = entry.contentRect.height;
            }
        };
        /**
         *  Handle anchor resize events
         */
        this.handleAnchorResize = (entry) => {
            this.anchorHeight = entry.contentRect.height;
            this.anchorWidth = entry.contentRect.width;
            if (this.verticalPosition === AnchoredRegionVerticalPositionLabel.top ||
                this.verticalPosition === AnchoredRegionVerticalPositionLabel.insetTop) {
                this.anchorBottom = this.anchorTop + this.anchorHeight;
            }
            else {
                this.anchorTop = this.anchorBottom - this.anchorHeight;
            }
            if (this.horizontalPosition === AnchoredRegionHorizontalPositionLabel.left ||
                this.horizontalPosition === AnchoredRegionHorizontalPositionLabel.insetLeft) {
                this.anchorRight = this.anchorLeft + this.anchorWidth;
            }
            else {
                this.anchorLeft = this.anchorRight - this.anchorWidth;
            }
        };
        /**
         *  Handle scroll events
         */
        this.handleScroll = () => {
            this.requestLayoutUpdate();
        };
        /**
         * Request's an animation frame if there are currently no open animation frame requests
         */
        this.requestLayoutUpdate = () => {
            if (this.openRequestAnimationFrame === false) {
                this.openRequestAnimationFrame = true;
                DOM.queueUpdate(this.updateLayout);
            }
        };
        /**
         *  Recalculate layout related state values
         */
        this.updateLayout = () => {
            this.openRequestAnimationFrame = false;
            if (this.viewportRect === null || this.positionerDimension === null) {
                return;
            }
            // if direction changes we need to reset the layout
            const newDirection = this.getDirection();
            if (newDirection !== this.currentDirection) {
                this.currentDirection = newDirection;
                this.reset();
                return;
            }
            this.updateForScrolling();
            let desiredVerticalPosition = AnchoredRegionVerticalPositionLabel.undefined;
            let desiredHorizontalPosition = AnchoredRegionHorizontalPositionLabel.undefined;
            if (this.horizontalPositioningMode !== "uncontrolled") {
                const horizontalOptions = this.getHorizontalPositioningOptions();
                if (this.horizontalDefaultPosition !== "unset") {
                    let dirCorrectedHorizontalDefaultPosition = this
                        .horizontalDefaultPosition;
                    if (dirCorrectedHorizontalDefaultPosition === "start" ||
                        dirCorrectedHorizontalDefaultPosition === "end") {
                        if (this.currentDirection === Direction.ltr) {
                            dirCorrectedHorizontalDefaultPosition =
                                dirCorrectedHorizontalDefaultPosition === "start"
                                    ? "left"
                                    : "right";
                        }
                        else {
                            dirCorrectedHorizontalDefaultPosition =
                                dirCorrectedHorizontalDefaultPosition === "start"
                                    ? "right"
                                    : "left";
                        }
                    }
                    switch (dirCorrectedHorizontalDefaultPosition) {
                        case "left":
                            desiredHorizontalPosition = this.horizontalInset
                                ? AnchoredRegionHorizontalPositionLabel.insetLeft
                                : AnchoredRegionHorizontalPositionLabel.left;
                            break;
                        case "right":
                            desiredHorizontalPosition = this.horizontalInset
                                ? AnchoredRegionHorizontalPositionLabel.insetRight
                                : AnchoredRegionHorizontalPositionLabel.right;
                            break;
                    }
                }
                const horizontalThreshold = this.horizontalThreshold !== undefined
                    ? Number(this.horizontalThreshold)
                    : this.positionerDimension.width;
                if (desiredHorizontalPosition ===
                    AnchoredRegionHorizontalPositionLabel.undefined ||
                    (!(this.horizontalPositioningMode === "locktodefault") &&
                        this.getAvailableWidth(desiredHorizontalPosition) <
                            horizontalThreshold)) {
                    desiredHorizontalPosition =
                        this.getAvailableWidth(horizontalOptions[0]) >
                            this.getAvailableWidth(horizontalOptions[1])
                            ? horizontalOptions[0]
                            : horizontalOptions[1];
                }
            }
            if (this.verticalPositioningMode !== "uncontrolled") {
                const verticalOptions = this.getVerticalPositioningOptions();
                if (this.verticalDefaultPosition !== "unset") {
                    switch (this.verticalDefaultPosition) {
                        case "top":
                            desiredVerticalPosition = this.verticalInset
                                ? AnchoredRegionVerticalPositionLabel.insetTop
                                : AnchoredRegionVerticalPositionLabel.top;
                            break;
                        case "bottom":
                            desiredVerticalPosition = this.verticalInset
                                ? AnchoredRegionVerticalPositionLabel.insetBottom
                                : AnchoredRegionVerticalPositionLabel.bottom;
                            break;
                    }
                }
                const verticalThreshold = this.verticalThreshold !== undefined
                    ? Number(this.verticalThreshold)
                    : this.positionerDimension.height;
                if (desiredVerticalPosition ===
                    AnchoredRegionVerticalPositionLabel.undefined ||
                    (!(this.verticalPositioningMode === "locktodefault") &&
                        this.getAvailableHeight(desiredVerticalPosition) < verticalThreshold)) {
                    desiredVerticalPosition =
                        this.getAvailableHeight(verticalOptions[0]) >
                            this.getAvailableHeight(verticalOptions[1])
                            ? verticalOptions[0]
                            : verticalOptions[1];
                }
            }
            const nextPositionerDimension = this.getNextRegionDimension(desiredHorizontalPosition, desiredVerticalPosition);
            const positionChanged = !(this.horizontalPosition === desiredHorizontalPosition &&
                this.verticalPosition === desiredVerticalPosition);
            this.setHorizontalPosition(desiredHorizontalPosition, nextPositionerDimension);
            this.setVerticalPosition(desiredVerticalPosition, nextPositionerDimension);
            this.transformOrigin = `${this.yTransformOrigin} ${this.xTransformOrigin}`;
            this.updateRegionStyle();
            this.initialLayoutComplete = true;
            if (positionChanged) {
                this.$emit("change");
            }
        };
        /**
         *  Updates the style string applied to the region element as well as the css classes attached
         *  to the root element
         */
        this.updateRegionStyle = () => {
            this.classList.toggle("horizontalInset", this.horizontalInset);
            this.classList.toggle("verticalInset", this.verticalInset);
            this.classList.toggle("top", this.verticalPosition === "top");
            this.classList.toggle("bottom", this.verticalPosition === "bottom");
            this.classList.toggle("left", this.horizontalPosition === "left");
            this.classList.toggle("right", this.horizontalPosition === "right");
            this.regionStyle = `
            height: ${this.regionHeight};
            width: ${this.regionWidth};
            top: ${this.regionTop};
            right: ${this.regionRight}; 
            bottom: ${this.regionBottom};
            left: ${this.regionLeft}; 
            transform-origin: ${this.transformOrigin};
            opacity: ${this.initialLayoutComplete ? 1 : 0}
        `;
        };
        /**
         * Get horizontal positioning state based on desired position
         */
        this.setHorizontalPosition = (desiredHorizontalPosition, nextPositionerDimension) => {
            let right = null;
            let left = null;
            let xTransformOrigin = Location.left;
            switch (desiredHorizontalPosition) {
                case AnchoredRegionHorizontalPositionLabel.left:
                    xTransformOrigin = Location.right;
                    right = nextPositionerDimension.width - this.baseHorizontalOffset;
                    break;
                case AnchoredRegionHorizontalPositionLabel.insetLeft:
                    xTransformOrigin = Location.right;
                    right =
                        nextPositionerDimension.width -
                            this.anchorWidth -
                            this.baseHorizontalOffset;
                    break;
                case AnchoredRegionHorizontalPositionLabel.insetRight:
                    xTransformOrigin = Location.left;
                    left = this.baseHorizontalOffset;
                    break;
                case AnchoredRegionHorizontalPositionLabel.right:
                    xTransformOrigin = Location.left;
                    left = this.anchorWidth + this.baseHorizontalOffset;
                    break;
            }
            this.xTransformOrigin = xTransformOrigin;
            this.regionRight = right === null ? "unset" : `${Math.floor(right).toString()}px`;
            this.regionLeft = left === null ? "unset" : `${Math.floor(left).toString()}px`;
            this.horizontalPosition = desiredHorizontalPosition;
            this.regionWidth = this.horizontalScaling
                ? `${Math.floor(nextPositionerDimension.width)}px`
                : "fit-content";
        };
        /**
         * Get vertical positioning state based on desired position
         */
        this.setVerticalPosition = (desiredVerticalPosition, nextPositionerDimension) => {
            let top = null;
            let bottom = null;
            let yTransformOrigin = Location.top;
            switch (desiredVerticalPosition) {
                case AnchoredRegionVerticalPositionLabel.top:
                    yTransformOrigin = Location.bottom;
                    bottom =
                        nextPositionerDimension.height +
                            this.anchorHeight -
                            this.baseVerticalOffset;
                    break;
                case AnchoredRegionVerticalPositionLabel.insetTop:
                    yTransformOrigin = Location.bottom;
                    bottom = nextPositionerDimension.height - this.baseVerticalOffset;
                    break;
                case AnchoredRegionVerticalPositionLabel.insetBottom:
                    yTransformOrigin = Location.top;
                    top = this.baseVerticalOffset - this.anchorHeight;
                    break;
                case AnchoredRegionVerticalPositionLabel.bottom:
                    yTransformOrigin = Location.top;
                    top = this.baseVerticalOffset;
                    break;
            }
            this.yTransformOrigin = yTransformOrigin;
            this.regionTop = top === null ? "unset" : `${Math.floor(top).toString()}px`;
            this.regionBottom =
                bottom === null ? "unset" : `${Math.floor(bottom).toString()}px`;
            this.verticalPosition = desiredVerticalPosition;
            this.regionHeight = this.verticalScaling
                ? `${Math.floor(nextPositionerDimension.height)}px`
                : "fit-content";
        };
        /**
         *  Update the offset values
         */
        this.updatePositionerOffset = (positionerRect) => {
            if (this.horizontalPositioningMode === "uncontrolled") {
                this.baseHorizontalOffset = this.anchorLeft - positionerRect.left;
            }
            else {
                switch (this.horizontalPosition) {
                    case AnchoredRegionHorizontalPositionLabel.undefined:
                        this.baseHorizontalOffset = this.anchorLeft - positionerRect.left;
                        break;
                    case AnchoredRegionHorizontalPositionLabel.left:
                        this.baseHorizontalOffset =
                            this.baseHorizontalOffset +
                                (this.anchorLeft - positionerRect.right);
                        break;
                    case AnchoredRegionHorizontalPositionLabel.insetLeft:
                        this.baseHorizontalOffset =
                            this.baseHorizontalOffset +
                                (this.anchorRight - positionerRect.right);
                        break;
                    case AnchoredRegionHorizontalPositionLabel.insetRight:
                        this.baseHorizontalOffset =
                            this.baseHorizontalOffset +
                                (this.anchorLeft - positionerRect.left);
                        break;
                    case AnchoredRegionHorizontalPositionLabel.right:
                        this.baseHorizontalOffset =
                            this.baseHorizontalOffset +
                                (this.anchorRight - positionerRect.left);
                        break;
                }
            }
            if (this.verticalPositioningMode === "uncontrolled") {
                this.baseVerticalOffset = this.anchorBottom - positionerRect.top;
            }
            else {
                switch (this.verticalPosition) {
                    case AnchoredRegionVerticalPositionLabel.undefined:
                        this.baseVerticalOffset = this.anchorBottom - positionerRect.top;
                        break;
                    case AnchoredRegionVerticalPositionLabel.top:
                        this.baseVerticalOffset =
                            this.baseVerticalOffset +
                                (this.anchorTop - positionerRect.bottom);
                        break;
                    case AnchoredRegionVerticalPositionLabel.insetTop:
                        this.baseVerticalOffset =
                            this.baseVerticalOffset +
                                (this.anchorBottom - positionerRect.bottom);
                        break;
                    case AnchoredRegionVerticalPositionLabel.insetBottom:
                        this.baseVerticalOffset =
                            this.baseVerticalOffset + (this.anchorTop - positionerRect.top);
                        break;
                    case AnchoredRegionVerticalPositionLabel.bottom:
                        this.baseVerticalOffset =
                            this.baseVerticalOffset +
                                (this.anchorBottom - positionerRect.top);
                        break;
                }
            }
        };
        /**
         * Check for scroll changes in viewport and adjust position data
         */
        this.updateForScrolling = () => {
            if (this.viewportElement === null || isNaN(this.viewportElement.scrollTop)) {
                return;
            }
            const scrollTop = this.viewportElement.scrollTop;
            const scrollLeft = RtlScrollConverter.getScrollLeft(this.viewportElement, this.currentDirection);
            if (this.viewportScrollTop !== scrollTop) {
                const verticalScrollDelta = this.viewportScrollTop - scrollTop;
                this.viewportScrollTop = scrollTop;
                this.anchorTop = this.anchorTop + verticalScrollDelta;
                this.anchorBottom = this.anchorBottom + verticalScrollDelta;
            }
            if (this.viewportScrollLeft !== scrollLeft) {
                const horizontalScrollDelta = this.viewportScrollLeft - scrollLeft;
                this.viewportScrollLeft = scrollLeft;
                this.anchorLeft =
                    this.currentDirection === Direction.ltr
                        ? this.anchorLeft + horizontalScrollDelta
                        : this.anchorLeft - horizontalScrollDelta;
                this.anchorRight =
                    this.currentDirection === Direction.ltr
                        ? this.anchorRight + horizontalScrollDelta
                        : this.anchorRight - horizontalScrollDelta;
            }
        };
        /**
         *  Get available Horizontal positions based on positioning mode
         */
        this.getHorizontalPositioningOptions = () => {
            if (this.horizontalInset) {
                return [
                    AnchoredRegionHorizontalPositionLabel.insetLeft,
                    AnchoredRegionHorizontalPositionLabel.insetRight,
                ];
            }
            return [
                AnchoredRegionHorizontalPositionLabel.left,
                AnchoredRegionHorizontalPositionLabel.right,
            ];
        };
        /**
         * Get available Vertical positions based on positioning mode
         */
        this.getVerticalPositioningOptions = () => {
            if (this.verticalInset) {
                return [
                    AnchoredRegionVerticalPositionLabel.insetTop,
                    AnchoredRegionVerticalPositionLabel.insetBottom,
                ];
            }
            return [
                AnchoredRegionVerticalPositionLabel.top,
                AnchoredRegionVerticalPositionLabel.bottom,
            ];
        };
        /**
         *  Get the width available for a particular horizontal position
         */
        this.getAvailableWidth = (positionOption) => {
            if (this.viewportRect !== null) {
                const spaceLeft = this.anchorLeft - this.viewportRect.left;
                const spaceRight = this.viewportRect.right - (this.anchorLeft + this.anchorWidth);
                switch (positionOption) {
                    case AnchoredRegionHorizontalPositionLabel.left:
                        return spaceLeft;
                    case AnchoredRegionHorizontalPositionLabel.insetLeft:
                        return spaceLeft + this.anchorWidth;
                    case AnchoredRegionHorizontalPositionLabel.insetRight:
                        return spaceRight + this.anchorWidth;
                    case AnchoredRegionHorizontalPositionLabel.right:
                        return spaceRight;
                }
            }
            return 0;
        };
        /**
         *  Get the height available for a particular vertical position
         */
        this.getAvailableHeight = (positionOption) => {
            if (this.viewportRect !== null) {
                const spaceAbove = this.anchorTop - this.viewportRect.top;
                const spaceBelow = this.viewportRect.bottom - (this.anchorTop + this.anchorHeight);
                switch (positionOption) {
                    case AnchoredRegionVerticalPositionLabel.top:
                        return spaceAbove;
                    case AnchoredRegionVerticalPositionLabel.insetTop:
                        return spaceAbove + this.anchorHeight;
                    case AnchoredRegionVerticalPositionLabel.insetBottom:
                        return spaceBelow + this.anchorHeight;
                    case AnchoredRegionVerticalPositionLabel.bottom:
                        return spaceBelow;
                }
            }
            return 0;
        };
        /**
         * Get region dimensions
         */
        this.getNextRegionDimension = (desiredHorizontalPosition, desiredVerticalPosition) => {
            const newRegionDimension = {
                height: this.positionerDimension.height,
                width: this.positionerDimension.width,
            };
            if (this.horizontalScaling) {
                newRegionDimension.width = this.getAvailableWidth(desiredHorizontalPosition);
            }
            if (this.verticalScaling) {
                newRegionDimension.height = this.getAvailableHeight(desiredVerticalPosition);
            }
            return newRegionDimension;
        };
        /**
         *  gets the current direction
         */
        this.getDirection = () => {
            if (this.viewportElement === null) {
                return Direction.ltr;
            }
            const closest = this.viewportElement.closest(`[${AnchoredRegion.DirectionAttributeName}]`);
            return closest === null ||
                closest.getAttribute(AnchoredRegion.DirectionAttributeName) === Direction.ltr
                ? Direction.ltr
                : Direction.rtl;
        };
        this.setInitialState();
    }
    anchorChanged() {
        if (this.initialLayoutComplete) {
            this.initialLayoutComplete = false;
            this.anchorElement = this.getAnchor();
            this.reset();
        }
    }
    viewportChanged() {
        if (this.initialLayoutComplete) {
            this.initialLayoutComplete = false;
            this.disconnectViewport(this.viewportElement);
            this.viewportElement = this.getViewport();
            this.reset();
        }
    }
    horizontalPositioningModeChanged() {
        this.updateLayoutForAttributeChange();
    }
    horizontalDefaultPositionChanged() {
        this.updateLayoutForAttributeChange();
    }
    horizontalInsetChanged() {
        this.updateLayoutForAttributeChange();
    }
    horizontalThresholdChanged() {
        this.updateLayoutForAttributeChange();
    }
    horizontalScalingChanged() {
        this.updateLayoutForAttributeChange();
    }
    verticalPositioningModeChanged() {
        this.updateLayoutForAttributeChange();
    }
    verticalDefaultPositionChanged() {
        this.updateLayoutForAttributeChange();
    }
    verticalInsetChanged() {
        this.updateLayoutForAttributeChange();
    }
    verticalThresholdChanged() {
        this.updateLayoutForAttributeChange();
    }
    verticalScalingChanged() {
        this.updateLayoutForAttributeChange();
    }
    anchorElementChanged() {
        if (this.initialLayoutComplete) {
            this.initialLayoutComplete = false;
            this.reset();
        }
    }
    viewportElementChanged() {
        if (this.initialLayoutComplete) {
            this.initialLayoutComplete = false;
            this.reset();
        }
    }
    connectedCallback() {
        super.connectedCallback();
        if (this.viewportElement === null) {
            this.viewportElement = this.getViewport();
        }
        if (this.anchorElement === null) {
            this.anchorElement = this.getAnchor();
        }
        if (this.anchorElement === null || this.viewportElement === null) {
            return;
        }
        this.currentDirection = this.getDirection();
        this.connectObservers();
    }
    disconnectedCallback() {
        super.disconnectedCallback();
        this.disconnectObservers();
    }
    adoptedCallback() {
        this.reset();
    }
    /**
     * event thrown when the region's position changes
     */
    updateLayoutForAttributeChange() {
        if (this.initialLayoutComplete) {
            this.requestLayoutUpdate();
        }
    }
    /**
     * resets the component
     */
    reset() {
        this.disconnectObservers();
        this.setInitialState();
        this.connectObservers();
    }
}
AnchoredRegion.DirectionAttributeName = "dir";
__decorate([
    attr
], AnchoredRegion.prototype, "anchor", void 0);
__decorate([
    attr
], AnchoredRegion.prototype, "viewport", void 0);
__decorate([
    attr({ attribute: "horizontal-positioning-mode" })
], AnchoredRegion.prototype, "horizontalPositioningMode", void 0);
__decorate([
    attr({ attribute: "horizontal-default-position" })
], AnchoredRegion.prototype, "horizontalDefaultPosition", void 0);
__decorate([
    attr({ attribute: "horizontal-inset", mode: "boolean" })
], AnchoredRegion.prototype, "horizontalInset", void 0);
__decorate([
    attr({ attribute: "horizontal-threshold" })
], AnchoredRegion.prototype, "horizontalThreshold", void 0);
__decorate([
    attr({ attribute: "horizontal-scaling", mode: "boolean" })
], AnchoredRegion.prototype, "horizontalScaling", void 0);
__decorate([
    attr({ attribute: "vertical-positioning-mode" })
], AnchoredRegion.prototype, "verticalPositioningMode", void 0);
__decorate([
    attr({ attribute: "vertical-default-position" })
], AnchoredRegion.prototype, "verticalDefaultPosition", void 0);
__decorate([
    attr({ attribute: "vertical-inset", mode: "boolean" })
], AnchoredRegion.prototype, "verticalInset", void 0);
__decorate([
    attr({ attribute: "vertical-threshold" })
], AnchoredRegion.prototype, "verticalThreshold", void 0);
__decorate([
    attr({ attribute: "vertical-scaling", mode: "boolean" })
], AnchoredRegion.prototype, "verticalScaling", void 0);
__decorate([
    observable
], AnchoredRegion.prototype, "regionStyle", void 0);
__decorate([
    observable
], AnchoredRegion.prototype, "initialLayoutComplete", void 0);
__decorate([
    observable
], AnchoredRegion.prototype, "anchorElement", void 0);
__decorate([
    observable
], AnchoredRegion.prototype, "viewportElement", void 0);
