import { __decorate } from "tslib";
import { attr, FASTElement } from "@microsoft/fast-element";
/**
 * Divider roles
 * @public
 */
export var DividerRole;
(function (DividerRole) {
    /**
     * The divider semantically separates content
     */
    DividerRole["separator"] = "separator";
    /**
     * The divider has no semantic value and is for visual presentation only.
     */
    DividerRole["presentation"] = "presentation";
})(DividerRole || (DividerRole = {}));
/**
 * A Divider Custom HTML Element.
 * Implements the {@link https://www.w3.org/TR/wai-aria-1.1/#separator | ARIA separator } or {@link https://www.w3.org/TR/wai-aria-1.1/#presentation | ARIA presentation}.
 *
 * @public
 */
export class Divider extends FASTElement {
    constructor() {
        super(...arguments);
        /**
         * The role of the element.
         *
         * @public
         * @defaultValue - {@link DividerRole.separator}
         * @remarks
         * HTML Attribute: role
         */
        this.role = DividerRole.separator;
    }
}
__decorate([
    attr
], Divider.prototype, "role", void 0);
