import { __decorate } from "tslib";
import { attr, booleanConverter, FASTElement } from "@microsoft/fast-element";
import { StartEnd } from "../patterns/start-end";
import { applyMixins } from "../utilities/apply-mixins";
/**
 * Menu items roles.
 * @public
 */
export var MenuItemRole;
(function (MenuItemRole) {
    MenuItemRole["menuitem"] = "menuitem";
    MenuItemRole["menuitemcheckbox"] = "menuitemcheckbox";
    MenuItemRole["menuitemradio"] = "menuitemradio";
})(MenuItemRole || (MenuItemRole = {}));
/**
 * A Switch Custom HTML Element.
 * Implements {@link https://www.w3.org/TR/wai-aria-1.1/#menuitem | ARIA menuitem }, {@link https://www.w3.org/TR/wai-aria-1.1/#menuitemcheckbox | ARIA menuitemcheckbox}, or {@link https://www.w3.org/TR/wai-aria-1.1/#menuitemradio | ARIA menuitemradio }.
 *
 * @public
 */
export class MenuItem extends FASTElement {
    constructor() {
        super(...arguments);
        /**
         * The expanded state of the element.
         *
         * @public
         * @remarks
         * HTML Attribute: aria-expanded
         */
        this.expanded = false;
        /**
         * The role of the element.
         *
         * @public
         * @remarks
         * HTML Attribute: role
         */
        this.role = MenuItemRole.menuitem;
        /**
         * @internal
         */
        this.handleMenuItemKeyDown = (e) => {
            this.change();
            return true;
        };
        /**
         * @internal
         */
        this.handleMenuItemClick = (e) => {
            this.change();
        };
        this.change = () => {
            this.$emit("change");
        };
    }
}
__decorate([
    attr({ mode: "boolean" })
], MenuItem.prototype, "disabled", void 0);
__decorate([
    attr({ attribute: "aria-expanded", mode: "reflect", converter: booleanConverter })
], MenuItem.prototype, "expanded", void 0);
__decorate([
    attr
], MenuItem.prototype, "role", void 0);
__decorate([
    attr
], MenuItem.prototype, "checked", void 0);
applyMixins(MenuItem, StartEnd);
